@@ -74,7 +74,11 @@ if [ -x /usr/bin/i686-pc-mingw32-gcc ]; then
     --without-qemu \
     --without-lxc \
     --without-uml \
+    --without-vbox \
     --without-openvz \
+    --without-one \
+    --without-phyp \
+    --without-netcf \
     --without-libvirtd
 
   make
