@@ -33,9 +33,9 @@
 
 typedef enum _esxVI_APIVersion esxVI_APIVersion;
 typedef enum _esxVI_ProductVersion esxVI_ProductVersion;
+typedef enum _esxVI_Occurrence esxVI_Occurrence;
 typedef struct _esxVI_Context esxVI_Context;
-typedef struct _esxVI_RemoteResponse esxVI_RemoteResponse;
-typedef struct _esxVI_RemoteRequest esxVI_RemoteRequest;
+typedef struct _esxVI_Response esxVI_Response;
 typedef struct _esxVI_Enumeration esxVI_Enumeration;
 typedef struct _esxVI_EnumerationValue esxVI_EnumerationValue;
 typedef struct _esxVI_List esxVI_List;
@@ -58,6 +58,14 @@ enum _esxVI_ProductVersion {
     esxVI_ProductVersion_VPX40
 };
 
+enum _esxVI_Occurrence {
+    esxVI_Occurrence_Undefined = 0,
+    esxVI_Occurrence_RequiredItem,
+    esxVI_Occurrence_OptionalItem,
+    esxVI_Occurrence_List,
+    esxVI_Occurrence_None
+};
+
 
 
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
@@ -66,6 +74,7 @@ enum _esxVI_ProductVersion {
 
 struct _esxVI_Context {
     char *url;
+    char *ipAddress;
     CURL *curl_handle;
     struct curl_slist *curl_headers;
     virMutex curl_lock;
@@ -84,64 +93,32 @@ struct _esxVI_Context {
 int esxVI_Context_Alloc(virConnectPtr conn, esxVI_Context **ctx);
 void esxVI_Context_Free(esxVI_Context **ctx);
 int esxVI_Context_Connect(virConnectPtr conn, esxVI_Context *ctx,
-                          const char *url, const char *username,
-                          const char *password, int noVerify);
-int esxVI_Context_Download(virConnectPtr conn, esxVI_Context *ctx,
-                           const char *url, char **content);
+                          const char *ipAddress, const char *url,
+                          const char *username, const char *password,
+                          int noVerify);
+int esxVI_Context_DownloadFile(virConnectPtr conn, esxVI_Context *ctx,
+                               const char *url, char **content);
+int esxVI_Context_UploadFile(virConnectPtr conn, esxVI_Context *ctx,
+                             const char *url, const char *content);
+int esxVI_Context_Execute(virConnectPtr conn, esxVI_Context *ctx,
+                          const char *methodName, const char *request,
+                          esxVI_Response **response, esxVI_Occurrence occurrence);
 
 
 
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
- * RemoteRequest
+ * Response
  */
 
-struct _esxVI_RemoteRequest {
-    char *request;                                    /* required */
-    char *xpathExpression;                            /* optional */
-};
-
-int esxVI_RemoteRequest_Alloc(virConnectPtr conn,
-                              esxVI_RemoteRequest **remoteRequest);
-void esxVI_RemoteRequest_Free(esxVI_RemoteRequest **remoteRequest);
-int esxVI_RemoteRequest_Execute(virConnectPtr conn, esxVI_Context *ctx,
-                                esxVI_RemoteRequest *remoteRequest,
-                                esxVI_RemoteResponse **remoteResponse);
-
-
-
-/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
- * RemoteResponse
- */
-
-struct _esxVI_RemoteResponse {
-    long response_code;                               /* required */
-    char *response;                                   /* required */
+struct _esxVI_Response {
+    int responseCode;                                 /* required */
+    char *content;                                    /* required */
     xmlDocPtr document;                               /* optional */
-    xmlXPathContextPtr xpathContext;                  /* optional */
-    xmlXPathObjectPtr xpathObject;                    /* optional */
+    xmlNodePtr node;                                  /* optional, list */
 };
 
-typedef int (*esxVI_RemoteResponse_DeserializeFunc) (virConnectPtr conn,
-                                                     xmlNodePtr node,
-                                                     void **item);
-typedef int (*esxVI_RemoteResponse_DeserializeListFunc) (virConnectPtr conn,
-                                                         xmlNodePtr node,
-                                                         esxVI_List **list);
-
-int esxVI_RemoteResponse_Alloc(virConnectPtr conn,
-                               esxVI_RemoteResponse **remoteResponse);
-void esxVI_RemoteResponse_Free(esxVI_RemoteResponse **remoteResponse);
-int esxVI_RemoteResponse_DeserializeXPathObject
-      (virConnectPtr conn, esxVI_RemoteResponse *remoteResponse,
-       esxVI_RemoteResponse_DeserializeFunc deserializeFunc, void **item);
-int esxVI_RemoteResponse_DeserializeXPathObjectList
-      (virConnectPtr conn, esxVI_RemoteResponse *remoteResponse,
-       esxVI_RemoteResponse_DeserializeListFunc deserializeListFunc,
-       esxVI_List **list);
-int esxVI_RemoteResponse_DeserializeXPathObjectAsManagedObjectReference
-    (virConnectPtr conn, esxVI_RemoteResponse *remoteResponse,
-     esxVI_ManagedObjectReference **managedObjectReference,
-     const char *expectedType);
+int esxVI_Response_Alloc(virConnectPtr conn, esxVI_Response **response);
+void esxVI_Response_Free(esxVI_Response **response);
 
 
 
@@ -161,7 +138,7 @@ struct _esxVI_Enumeration {
 
 int esxVI_Enumeration_CastFromAnyType(virConnectPtr conn,
                                       const esxVI_Enumeration *enumeration,
-                                      esxVI_AnyType *anyType, int *boolean);
+                                      esxVI_AnyType *anyType, int *value);
 int esxVI_Enumeration_Serialize(virConnectPtr conn,
                                 const esxVI_Enumeration *enumeration,
                                 int value, const char *element,
@@ -183,6 +160,9 @@ struct _esxVI_List {
 typedef int (*esxVI_List_FreeFunc) (esxVI_List **item);
 typedef int (*esxVI_List_DeepCopyFunc) (virConnectPtr conn, esxVI_List **dest,
                                         esxVI_List *src);
+typedef int (*esxVI_List_CastFromAnyTypeFunc) (virConnectPtr conn,
+                                               esxVI_AnyType *anyType,
+                                               esxVI_List **item);
 typedef int (*esxVI_List_SerializeFunc) (virConnectPtr conn, esxVI_List *item,
                                          const char *element,
                                          virBufferPtr output,
@@ -195,6 +175,10 @@ int esxVI_List_DeepCopy(virConnectPtr conn, esxVI_List **destList,
                         esxVI_List *srcList,
                         esxVI_List_DeepCopyFunc deepCopyFunc,
                         esxVI_List_FreeFunc freeFunc);
+int esxVI_List_CastFromAnyType(virConnectPtr conn, esxVI_AnyType *anyType,
+                               esxVI_List **list,
+                               esxVI_List_CastFromAnyTypeFunc castFromAnyTypeFunc,
+                               esxVI_List_FreeFunc freeFunc);
 int esxVI_List_Serialize(virConnectPtr conn, esxVI_List *list,
                          const char *element, virBufferPtr output,
                          esxVI_Boolean required,
@@ -208,35 +192,48 @@ int esxVI_List_Deserialize(virConnectPtr conn, xmlNodePtr node,
 
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
  * Utility and Convenience Functions
+ *
+ * Function naming scheme:
+ *  - 'lookup' functions query the ESX or vCenter for information
+ *  - 'get' functions get information from a local object
  */
 
-int
-esxVI_Alloc(virConnectPtr conn, void **ptrptr, size_t size);
+int esxVI_Alloc(virConnectPtr conn, void **ptrptr, size_t size);
 
-int
-esxVI_CheckSerializationNecessity(virConnectPtr conn, const char *element,
-                                  esxVI_Boolean required);
+int esxVI_CheckSerializationNecessity(virConnectPtr conn, const char *element,
+                                      esxVI_Boolean required);
 
 int esxVI_BuildFullTraversalSpecItem
       (virConnectPtr conn, esxVI_SelectionSpec **fullTraversalSpecList,
        const char *name, const char *type, const char *path,
        const char *selectSetNames);
+
 int esxVI_BuildFullTraversalSpecList
       (virConnectPtr conn, esxVI_SelectionSpec **fullTraversalSpecList);
 
 int esxVI_EnsureSession(virConnectPtr conn, esxVI_Context *ctx);
 
-int esxVI_GetObjectContent(virConnectPtr conn, esxVI_Context *ctx,
-                           esxVI_ManagedObjectReference *root,
-                           const char *type, esxVI_String *propertyNameList,
-                           esxVI_Boolean recurse,
-                           esxVI_ObjectContent **objectContentList);
+int esxVI_LookupObjectContentByType(virConnectPtr conn, esxVI_Context *ctx,
+                                    esxVI_ManagedObjectReference *root,
+                                    const char *type,
+                                    esxVI_String *propertyNameList,
+                                    esxVI_Boolean recurse,
+                                    esxVI_ObjectContent **objectContentList);
+
+int esxVI_GetManagedEntityStatus
+      (virConnectPtr conn, esxVI_ObjectContent *objectContent,
+       const char *propertyName,
+       esxVI_ManagedEntityStatus *managedEntityStatus);
 
 int esxVI_GetVirtualMachinePowerState
       (virConnectPtr conn, esxVI_ObjectContent *virtualMachine,
        esxVI_VirtualMachinePowerState *powerState);
 
-int esxVI_GetNumberOfDomainsByPowerState
+int esxVI_GetVirtualMachineQuestionInfo
+      (virConnectPtr conn, esxVI_ObjectContent *virtualMachine,
+       esxVI_VirtualMachineQuestionInfo **questionInfo);
+
+int esxVI_LookupNumberOfDomainsByPowerState
       (virConnectPtr conn, esxVI_Context *ctx,
        esxVI_VirtualMachinePowerState powerState, esxVI_Boolean inverse);
 
@@ -244,14 +241,45 @@ int esxVI_GetVirtualMachineIdentity(virConnectPtr conn,
                                     esxVI_ObjectContent *virtualMachine,
                                     int *id, char **name, unsigned char *uuid);
 
+int esxVI_LookupResourcePoolByHostSystem
+      (virConnectPtr conn, esxVI_Context *ctx, esxVI_ObjectContent *hostSystem,
+       esxVI_ManagedObjectReference **resourcePool);
+
 int esxVI_LookupHostSystemByIp(virConnectPtr conn, esxVI_Context *ctx,
-                               const char *ip, esxVI_String *propertyNameList,
+                               const char *ipAddress,
+                               esxVI_String *propertyNameList,
                                esxVI_ObjectContent **hostSystem);
 
 int esxVI_LookupVirtualMachineByUuid(virConnectPtr conn, esxVI_Context *ctx,
                                      const unsigned char *uuid,
                                      esxVI_String *propertyNameList,
-                                     esxVI_ObjectContent **virtualMachine);
+                                     esxVI_ObjectContent **virtualMachine,
+                                     esxVI_Occurrence occurrence);
+
+int esxVI_LookupVirtualMachineByUuidAndPrepareForTask
+      (virConnectPtr conn, esxVI_Context *ctx, const unsigned char *uuid,
+       esxVI_String *propertyNameList, esxVI_ObjectContent **virtualMachine,
+       esxVI_Boolean autoAnswer);
+
+int esxVI_LookupDatastoreByName(virConnectPtr conn, esxVI_Context *ctx,
+                                const char *name,
+                                esxVI_String *propertyNameList,
+                                esxVI_ObjectContent **datastore,
+                                esxVI_Occurrence occurrence);
+
+int esxVI_LookupTaskInfoByTask(virConnectPtr conn, esxVI_Context *ctx,
+                               esxVI_ManagedObjectReference *task,
+                               esxVI_TaskInfo **taskInfo);
+
+int esxVI_LookupPendingTaskInfoListByVirtualMachine
+      (virConnectPtr conn, esxVI_Context *ctx,
+       esxVI_ObjectContent *virtualMachine,
+       esxVI_TaskInfo **pendingTaskInfoList);
+
+int esxVI_LookupAndHandleVirtualMachineQuestion(virConnectPtr conn,
+                                                esxVI_Context *ctx,
+                                                const unsigned char *uuid,
+                                                esxVI_Boolean autoAnswer);
 
 int esxVI_StartVirtualMachineTask(virConnectPtr conn, esxVI_Context *ctx,
                                   const char *name, const char *request,
@@ -266,8 +294,16 @@ int esxVI_SimpleVirtualMachineMethod
       (virConnectPtr conn, esxVI_Context *ctx, const char *name,
        esxVI_ManagedObjectReference *virtualMachine);
 
+int esxVI_HandleVirtualMachineQuestion
+      (virConnectPtr conn, esxVI_Context *ctx,
+       esxVI_ManagedObjectReference *virtualMachine,
+       esxVI_VirtualMachineQuestionInfo *questionInfo,
+       esxVI_Boolean autoAnswer);
+
 int esxVI_WaitForTaskCompletion(virConnectPtr conn, esxVI_Context *ctx,
                                 esxVI_ManagedObjectReference *task,
+                                const unsigned char *virtualMachineUuid,
+                                esxVI_Boolean autoAnswer,
                                 esxVI_TaskInfoState *finalState);
 
 #endif /* __ESX_VI_H__ */
