@@ -52,6 +52,7 @@ typedef struct _esxVI_DateTime esxVI_DateTime;
  * VI Enums
  */
 
+typedef enum _esxVI_ManagedEntityStatus esxVI_ManagedEntityStatus;
 typedef enum _esxVI_ObjectUpdateKind esxVI_ObjectUpdateKind;
 typedef enum _esxVI_PerfSummaryType esxVI_PerfSummaryType;
 typedef enum _esxVI_PerfStatsType esxVI_PerfStatsType;
@@ -70,6 +71,7 @@ typedef enum _esxVI_VirtualMachinePowerState esxVI_VirtualMachinePowerState;
 typedef struct _esxVI_Fault esxVI_Fault;
 typedef struct _esxVI_ManagedObjectReference esxVI_ManagedObjectReference;
 typedef struct _esxVI_DynamicProperty esxVI_DynamicProperty;
+typedef struct _esxVI_HostCpuIdInfo esxVI_HostCpuIdInfo;
 typedef struct _esxVI_SelectionSpec esxVI_SelectionSpec;
 typedef struct _esxVI_TraversalSpec esxVI_TraversalSpec;
 typedef struct _esxVI_ObjectSpec esxVI_ObjectSpec;
@@ -84,16 +86,20 @@ typedef struct _esxVI_ServiceContent esxVI_ServiceContent;
 typedef struct _esxVI_UpdateSet esxVI_UpdateSet;
 typedef struct _esxVI_SharesInfo esxVI_SharesInfo;
 typedef struct _esxVI_ResourceAllocationInfo esxVI_ResourceAllocationInfo;
+typedef struct _esxVI_ResourcePoolResourceUsage esxVI_ResourcePoolResourceUsage;
 typedef struct _esxVI_VirtualMachineConfigSpec esxVI_VirtualMachineConfigSpec;
 typedef struct _esxVI_Event esxVI_Event;
 typedef struct _esxVI_UserSession esxVI_UserSession;
+typedef struct _esxVI_VirtualMachineQuestionInfo esxVI_VirtualMachineQuestionInfo;
 typedef struct _esxVI_ElementDescription esxVI_ElementDescription;
+typedef struct _esxVI_ChoiceOption esxVI_ChoiceOption;
 typedef struct _esxVI_PerfMetricId esxVI_PerfMetricId;
 typedef struct _esxVI_PerfCounterInfo esxVI_PerfCounterInfo;
 typedef struct _esxVI_PerfQuerySpec esxVI_PerfQuerySpec;
 typedef struct _esxVI_PerfSampleInfo esxVI_PerfSampleInfo;
 typedef struct _esxVI_PerfMetricIntSeries esxVI_PerfMetricIntSeries;
 typedef struct _esxVI_PerfEntityMetric esxVI_PerfEntityMetric;
+typedef struct _esxVI_TaskInfo esxVI_TaskInfo;
 
 
 
@@ -125,11 +131,11 @@ enum _esxVI_Boolean {
     esxVI_Boolean_False,
 };
 
-int esxVI_Boolean_Serialize(virConnectPtr conn, esxVI_Boolean boolean,
+int esxVI_Boolean_Serialize(virConnectPtr conn, esxVI_Boolean boolean_,
                             const char *element, virBufferPtr output,
                             esxVI_Boolean required);
 int esxVI_Boolean_Deserialize(virConnectPtr conn, xmlNodePtr node,
-                              esxVI_Boolean *boolean);
+                              esxVI_Boolean *boolean_);
 
 
 
@@ -276,6 +282,24 @@ int esxVI_DateTime_Deserialize(virConnectPtr conn, xmlNodePtr node,
 
 
 
+/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
+ * VI Enum: ManagedEntityStatus
+ */
+
+enum _esxVI_ManagedEntityStatus {
+    esxVI_ManagedEntityStatus_Undefined = 0,
+    esxVI_ManagedEntityStatus_Gray,
+    esxVI_ManagedEntityStatus_Green,
+    esxVI_ManagedEntityStatus_Yellow,
+    esxVI_ManagedEntityStatus_Red,
+};
+
+int esxVI_ManagedEntityStatus_CastFromAnyType
+      (virConnectPtr conn, esxVI_AnyType *anyType,
+       esxVI_ManagedEntityStatus *managedEntityStatus);
+
+
+
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
  * VI Enum: ObjectUpdateKind
  */
@@ -382,6 +406,8 @@ enum _esxVI_TaskInfoState {
 int esxVI_TaskInfoState_CastFromAnyType(virConnectPtr conn,
                                         esxVI_AnyType *anyType,
                                         esxVI_TaskInfoState *taskInfoState);
+int esxVI_TaskInfoState_Deserialize(virConnectPtr conn, xmlNodePtr node,
+                                    esxVI_TaskInfoState *taskInfoState);
 
 
 
@@ -459,11 +485,19 @@ void esxVI_ManagedObjectReference_Free
 int esxVI_ManagedObjectReference_DeepCopy(virConnectPtr conn,
                                           esxVI_ManagedObjectReference **dest,
                                           esxVI_ManagedObjectReference *src);
+int esxVI_ManagedObjectReference_AppendToList
+      (virConnectPtr conn,
+       esxVI_ManagedObjectReference **managedObjectReferenceList,
+       esxVI_ManagedObjectReference *managedObjectReference);
 int esxVI_ManagedObjectReference_CastFromAnyType(virConnectPtr conn,
                                                  esxVI_AnyType *anyType,
                                                  esxVI_ManagedObjectReference
                                                  **managedObjectReference,
                                                  const char *expectedType);
+int esxVI_ManagedObjectReference_CastListFromAnyType
+      (virConnectPtr conn, esxVI_AnyType *anyType,
+       esxVI_ManagedObjectReference **managedObjectReferenceList,
+       const char *expectedType);
 int esxVI_ManagedObjectReference_Serialize
       (virConnectPtr conn,
        esxVI_ManagedObjectReference *managedObjectReference,
@@ -507,7 +541,39 @@ int esxVI_DynamicProperty_Deserialize(virConnectPtr conn, xmlNodePtr node,
                                       esxVI_DynamicProperty **dynamicProperty);
 int esxVI_DynamicProperty_DeserializeList
       (virConnectPtr conn, xmlNodePtr node,
-       esxVI_DynamicProperty **dynamicProperty);
+       esxVI_DynamicProperty **dynamicPropertyList);
+
+
+
+/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
+ * VI Type: HostCpuIdInfo
+ */
+
+struct _esxVI_HostCpuIdInfo {
+    esxVI_HostCpuIdInfo *_next;                            /* optional */
+
+    esxVI_Int *level;                                      /* required */
+    char *vendor;                                          /* optional */
+    char *eax;                                             /* optional */
+    char *ebx;                                             /* optional */
+    char *ecx;                                             /* optional */
+    char *edx;                                             /* optional */
+};
+
+int esxVI_HostCpuIdInfo_Alloc(virConnectPtr conn,
+                              esxVI_HostCpuIdInfo **hostCpuIdInfo);
+void esxVI_HostCpuIdInfo_Free(esxVI_HostCpuIdInfo **hostCpuIdInfoList);
+int esxVI_HostCpuIdInfo_CastFromAnyType(virConnectPtr conn,
+                                        esxVI_AnyType *anyType,
+                                        esxVI_HostCpuIdInfo **hostCpuIdInfo);
+int esxVI_HostCpuIdInfo_CastListFromAnyType
+      (virConnectPtr conn, esxVI_AnyType *anyType,
+       esxVI_HostCpuIdInfo **hostCpuIdInfoList);
+int esxVI_HostCpuIdInfo_Deserialize(virConnectPtr conn, xmlNodePtr node,
+                                    esxVI_HostCpuIdInfo **hostCpuIdInfo);
+int esxVI_HostCpuIdInfo_DeserializeList
+      (virConnectPtr conn, xmlNodePtr node,
+       esxVI_HostCpuIdInfo **hostCpuIdInfoList);
 
 
 
@@ -612,7 +678,7 @@ int esxVI_PropertyChange_Deserialize(virConnectPtr conn, xmlNodePtr node,
                                      esxVI_PropertyChange **propertyChange);
 int esxVI_PropertyChange_DeserializeList
       (virConnectPtr conn, xmlNodePtr node,
-       esxVI_PropertyChange **propertyChange);
+       esxVI_PropertyChange **propertyChangeList);
 
 
 
@@ -882,6 +948,33 @@ int esxVI_ResourceAllocationInfo_Serialize
 
 
 
+/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
+ * VI Type: ResourcePoolResourceUsage
+ */
+
+struct _esxVI_ResourcePoolResourceUsage {
+    esxVI_Long *reservationUsed;                           /* required */
+    esxVI_Long *reservationUsedForVm;                      /* required */
+    esxVI_Long *unreservedForPool;                         /* required */
+    esxVI_Long *unreservedForVm;                           /* required */
+    esxVI_Long *overallUsage;                              /* required */
+    esxVI_Long *maxUsage;                                  /* required */
+};
+
+int esxVI_ResourcePoolResourceUsage_Alloc
+      (virConnectPtr conn,
+       esxVI_ResourcePoolResourceUsage **resourcePoolResourceUsage);
+void esxVI_ResourcePoolResourceUsage_Free
+       (esxVI_ResourcePoolResourceUsage **resourcePoolResourceUsage);
+int esxVI_ResourcePoolResourceUsage_CastFromAnyType
+      (virConnectPtr conn, esxVI_AnyType *anyType,
+       esxVI_ResourcePoolResourceUsage **resourcePoolResourceUsage);
+int esxVI_ResourcePoolResourceUsage_Deserialize
+      (virConnectPtr conn, xmlNodePtr node,
+       esxVI_ResourcePoolResourceUsage **resourcePoolResourceUsage);
+
+
+
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
  * VI Type: VirtualMachineConfigSpec
  */
@@ -984,6 +1077,32 @@ int esxVI_UserSession_Deserialize(virConnectPtr conn, xmlNodePtr node,
 
 
 
+/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
+ * VI Type: VirtualMachineQuestionInfo
+ */
+
+/* FIXME: implement the rest */
+struct _esxVI_VirtualMachineQuestionInfo {
+    char *id;                                              /* required */
+    char *text;                                            /* required */
+    esxVI_ChoiceOption *choice;                            /* required */
+    /*esxVI_VirtualMachineMessage *message;*/              /* optional, list */
+};
+
+int esxVI_VirtualMachineQuestionInfo_Alloc
+      (virConnectPtr conn,
+       esxVI_VirtualMachineQuestionInfo **virtualMachineQuestionInfo);
+void esxVI_VirtualMachineQuestionInfo_Free
+       (esxVI_VirtualMachineQuestionInfo **virtualMachineQuestionInfo);
+int esxVI_VirtualMachineQuestionInfo_CastFromAnyType
+      (virConnectPtr conn, esxVI_AnyType *anyType,
+       esxVI_VirtualMachineQuestionInfo **virtualMachineQuestionInfo);
+int esxVI_VirtualMachineQuestionInfo_Deserialize
+      (virConnectPtr conn, xmlNodePtr node,
+       esxVI_VirtualMachineQuestionInfo **virtualMachineQuestionInfo);
+
+
+
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
  * VI Type: ElementDescription extends Description
  *
@@ -1007,12 +1126,40 @@ int esxVI_ElementDescription_Alloc
       (virConnectPtr conn, esxVI_ElementDescription **elementDescription);
 void esxVI_ElementDescription_Free
        (esxVI_ElementDescription **elementDescription);
+int esxVI_ElementDescription_AppendToList
+      (virConnectPtr conn, esxVI_ElementDescription **elementDescriptionList,
+       esxVI_ElementDescription *elementDescription);
 int esxVI_ElementDescription_Deserialize
       (virConnectPtr conn, xmlNodePtr node,
        esxVI_ElementDescription **elementDescription);
 
 
 
+/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
+ * VI Type: ChoiceOption extends OptionType
+ *
+ *          In contrast to SelectionSpec and TraversalSpec just merge
+ *          OptionType into ChoiceOption for simplicity, because
+ *          only ChoiceOption is used.
+ */
+
+struct _esxVI_ChoiceOption {
+    /* OptionType */
+    esxVI_Boolean valueIsReadonly;                         /* optional */
+
+    /* ChoiceOption */
+    esxVI_ElementDescription *choiceInfo;                  /* required, list */
+    esxVI_Int *defaultIndex;                               /* optional */
+};
+
+int esxVI_ChoiceOption_Alloc(virConnectPtr conn,
+                             esxVI_ChoiceOption **choiceOption);
+void esxVI_ChoiceOption_Free(esxVI_ChoiceOption **choiceOption);
+int esxVI_ChoiceOption_Deserialize(virConnectPtr conn, xmlNodePtr node,
+                                   esxVI_ChoiceOption **choiceOption);
+
+
+
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
  * VI Type: PerfMetricId
  */
@@ -1077,13 +1224,13 @@ int esxVI_PerfCounterInfo_DeserializeList
 struct _esxVI_PerfQuerySpec {
     esxVI_PerfQuerySpec *_next;                            /* optional */
 
-    esxVI_ManagedObjectReference* entity;                  /* required */
-    esxVI_DateTime* startTime;                             /* optional */
-    esxVI_DateTime* endTime;                               /* optional */
+    esxVI_ManagedObjectReference *entity;                  /* required */
+    esxVI_DateTime *startTime;                             /* optional */
+    esxVI_DateTime *endTime;                               /* optional */
     esxVI_Int *maxSample;                                  /* optional */
     esxVI_PerfMetricId *metricId;                          /* optional, list */
     esxVI_Int *intervalId;                                 /* optional */
-    char* format;                                          /* optional */ // FIXME: see PerfFormat
+    char *format;                                          /* optional */ // FIXME: see PerfFormat
 };
 
 int esxVI_PerfQuerySpec_Alloc(virConnectPtr conn,
@@ -1107,8 +1254,8 @@ int esxVI_PerfQuerySpec_SerializeList(virConnectPtr conn,
 struct _esxVI_PerfSampleInfo {
     esxVI_PerfSampleInfo *_next;                           /* optional */
 
-    esxVI_DateTime* timestamp;                             /* required */
-    esxVI_Int* interval;                                   /* required */
+    esxVI_DateTime *timestamp;                             /* required */
+    esxVI_Int *interval;                                   /* required */
 };
 
 int esxVI_PerfSampleInfo_Alloc(virConnectPtr conn,
@@ -1192,4 +1339,43 @@ int esxVI_PerfEntityMetric_DeserializeList
       (virConnectPtr conn, xmlNodePtr node,
        esxVI_PerfEntityMetric **perfEntityMetricList);
 
+
+
+/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
+ * VI Type: TaskInfo
+ */
+
+struct _esxVI_TaskInfo {
+    esxVI_TaskInfo *_next;                                 /* optional */
+
+    char *key;                                             /* required */
+    esxVI_ManagedObjectReference *task;                    /* required */
+    char *name;                                            /* optional */
+    char *descriptionId;                                   /* required */
+    esxVI_ManagedObjectReference *entity;                  /* optional */
+    char *entityName;                                      /* optional */
+    /*esxVI_ManagedObjectReference *locked;*/                  /* optional, list *//* FIXME */
+    esxVI_TaskInfoState state;                             /* required */
+    esxVI_Boolean cancelled;                               /* required */
+    esxVI_Boolean cancelable;                              /* required */
+    /*esxVI_MethodFault *error;*/                              /* optional *//* FIXME */
+    esxVI_AnyType *result;                                 /* optional */
+    esxVI_Int *progress;                                   /* optional */
+    /*esxVI_TaskReason *reason;*/                              /* required *//* FIXME */
+    esxVI_DateTime *queueTime;                             /* required */
+    esxVI_DateTime *startTime;                             /* optional */
+    esxVI_DateTime *completeTime;                          /* optional */
+    esxVI_Int *eventChainId;                               /* required */
+};
+
+int esxVI_TaskInfo_Alloc(virConnectPtr conn, esxVI_TaskInfo **taskInfo);
+void esxVI_TaskInfo_Free(esxVI_TaskInfo **taskInfoList);
+int esxVI_TaskInfo_CastFromAnyType(virConnectPtr conn, esxVI_AnyType *anyType,
+                                   esxVI_TaskInfo **taskInfo);
+int esxVI_TaskInfo_AppendToList(virConnectPtr conn,
+                                esxVI_TaskInfo **taskInfoList,
+                                esxVI_TaskInfo *taskInfo);
+int esxVI_TaskInfo_Deserialize(virConnectPtr conn, xmlNodePtr node,
+                               esxVI_TaskInfo **taskInfo);
+
 #endif /* __ESX_VI_TYPES_H__ */
