@@ -6,7 +6,7 @@
 
 #include "stats_linux.h"
 #include "internal.h"
-
+#include "xen/block_stats.h"
 #include "testutils.h"
 
 #if WITH_XEN
@@ -25,7 +25,7 @@ static int testDevice(const char *path, int expect)
     if (actual == expect) {
         return 0;
     } else {
-        if (getenv("DEBUG_TESTS"))
+        if (virtTestGetDebug())
             fprintf(stderr, "Expect %-6d Actual %-6d\n", expect, actual);
         return -1;
     }
@@ -55,7 +55,7 @@ mymain(int argc ATTRIBUTE_UNUSED,
      * register a handler to stop error messages cluttering
      * up display
      */
-    if (!getenv("VIR_TEST_DEBUG"))
+    if (!virtTestGetDebug())
         virSetErrorFunc(NULL, testQuietError);
 
 #define DO_TEST(dev, num)                                              \
