@@ -4,6 +4,54 @@
 #include <stdlib.h>
 
 #include "testutilsqemu.h"
+#include "testutils.h"
+#include "memory.h"
+
+static virCapsGuestMachinePtr *testQemuAllocMachines(int *nmachines)
+{
+    virCapsGuestMachinePtr *machines;
+    static const char *const x86_machines[] = {
+        "pc", "isapc"
+    };
+
+    machines = virCapabilitiesAllocMachines(x86_machines,
+                                            ARRAY_CARDINALITY(x86_machines));
+    if (machines == NULL)
+        return NULL;
+
+    *nmachines = ARRAY_CARDINALITY(x86_machines);
+
+    return machines;
+}
+
+/* Newer versions of qemu have versioned machine types to allow
+ * compatibility with older releases.
+ * The 'pc' machine type is an alias of the newest machine type.
+ */
+static virCapsGuestMachinePtr *testQemuAllocNewerMachines(int *nmachines)
+{
+    virCapsGuestMachinePtr *machines;
+    char *canonical;
+    static const char *const x86_machines[] = {
+        "pc-0.11", "pc", "pc-0.10", "isapc"
+    };
+
+    if ((canonical = strdup(x86_machines[0])) == NULL)
+        return NULL;
+
+    machines = virCapabilitiesAllocMachines(x86_machines,
+                                            ARRAY_CARDINALITY(x86_machines));
+    if (machines == NULL) {
+        VIR_FREE(canonical);
+        return NULL;
+    }
+
+    machines[1]->canonical = canonical;
+
+    *nmachines = ARRAY_CARDINALITY(x86_machines);
+
+    return machines;
+}
 
 virCapsPtr testQemuCapsInit(void) {
     struct utsname utsname;
@@ -11,9 +59,6 @@ virCapsPtr testQemuCapsInit(void) {
     virCapsGuestPtr guest;
     virCapsGuestMachinePtr *machines;
     int nmachines;
-    static const char *const x86_machines[] = {
-        "pc", "isapc"
-    };
     static const char *const xen_machines[] = {
         "xenner"
     };
@@ -23,8 +68,7 @@ virCapsPtr testQemuCapsInit(void) {
                                    0, 0)) == NULL)
         return NULL;
 
-    nmachines = 2;
-    if ((machines = virCapabilitiesAllocMachines(x86_machines, nmachines)) == NULL)
+    if ((machines = testQemuAllocMachines(&nmachines)) == NULL)
         goto cleanup;
 
     if ((guest = virCapabilitiesAddGuest(caps, "hvm", "i686", 32,
@@ -41,8 +85,7 @@ virCapsPtr testQemuCapsInit(void) {
                                       NULL) == NULL)
         goto cleanup;
 
-    nmachines = 2;
-    if ((machines = virCapabilitiesAllocMachines(x86_machines, nmachines)) == NULL)
+    if ((machines = testQemuAllocNewerMachines(&nmachines)) == NULL)
         goto cleanup;
 
     if ((guest = virCapabilitiesAddGuest(caps, "hvm", "x86_64", 64,
@@ -58,21 +101,26 @@ virCapsPtr testQemuCapsInit(void) {
                                       0,
                                       NULL) == NULL)
         goto cleanup;
+
+    if ((machines = testQemuAllocMachines(&nmachines)) == NULL)
+        goto cleanup;
+
     if (virCapabilitiesAddGuestDomain(guest,
                                       "kvm",
                                       "/usr/bin/kvm",
                                       NULL,
-                                      0,
-                                      NULL) == NULL)
+                                      nmachines,
+                                      machines) == NULL)
         goto cleanup;
+    machines = NULL;
 
-    nmachines = 1;
+    nmachines = ARRAY_CARDINALITY(xen_machines);
     if ((machines = virCapabilitiesAllocMachines(xen_machines, nmachines)) == NULL)
         goto cleanup;
 
     if ((guest = virCapabilitiesAddGuest(caps, "xen", "x86_64", 64,
                                          "/usr/bin/xenner", NULL,
-                                         1, machines)) == NULL)
+                                         nmachines, machines)) == NULL)
         goto cleanup;
     machines = NULL;
 
@@ -84,6 +132,18 @@ virCapsPtr testQemuCapsInit(void) {
                                       NULL) == NULL)
         goto cleanup;
 
+    if (virtTestGetDebug()) {
+        char *caps_str;
+
+        caps_str = virCapabilitiesFormatXML(caps);
+        if (!caps_str)
+            goto cleanup;
+
+        fprintf(stderr, "QEMU driver capabilities:\n%s", caps_str);
+
+        VIR_FREE(caps_str);
+    }
+
     return caps;
 
 cleanup:
